"""Template tags and filters for FRED objects."""
from typing import Dict, cast

from django import template

from ..constants import DnskeyAlgorithm, DnskeyFlag, DnskeyProtocol
from ..settings import SECRETARY_SETTINGS

register = template.Library()


@register.filter
def parse_dnskey_algorithm(value: str) -> DnskeyAlgorithm:  # type: ignore[valid-type]
    """Return parsed DNSKEY algorithm."""
    return DnskeyAlgorithm(value)  # type: ignore[operator, no-any-return]


@register.filter
def parse_dnskey_flags(value: str) -> DnskeyFlag:  # type: ignore[valid-type]
    """Return parsed DNSKEY flags."""
    return DnskeyFlag(value)  # type: ignore[operator, no-any-return]


@register.filter
def parse_dnskey_protocol(value: str) -> DnskeyProtocol:  # type: ignore[valid-type]
    """Return parsed DNSKEY algorithm."""
    return DnskeyProtocol(value)  # type: ignore[operator, no-any-return]


@register.filter
def state_flag_description(value: str) -> str:
    """Return description of the state flag."""
    return cast(Dict, SECRETARY_SETTINGS.STATE_FLAGS_DESCRIPTIONS).get(value, value)
