"""Template tags and filters for parsing."""
from datetime import date, datetime

from django import template
from django.utils.dateparse import parse_date as _parse_date, parse_datetime as _parse_datetime

register = template.Library()


@register.filter
def parse_date(value: str) -> date:
    """Parse date from a string."""
    parsed = _parse_date(value)
    if parsed is None:
        raise ValueError("{!r} is not a valid date.".format(value))
    return parsed


@register.filter
def parse_datetime(value: str) -> datetime:
    """Parse datetime from a string."""
    parsed = _parse_datetime(value)
    if parsed is None:
        raise ValueError("{!r} is not a valid datetime.".format(value))
    return parsed
