from datetime import date, datetime

import django
from django.test import SimpleTestCase
from pytz import UTC

from django_secretary.templatetags.parsers import parse_date, parse_datetime


class ParseDateTest(SimpleTestCase):
    def test_valid(self):
        data = (
            # value, result
            ('2001-02-03', date(2001, 2, 3)),
        )
        for value, result in data:
            with self.subTest(value=value):
                self.assertEqual(parse_date(value), result)

    def test_invalid(self):
        data = (
            # value, error
            ('', 'not a valid date'),
            ('JUNK', 'not a valid date'),
            ('2001-13-03', 'month must be in 1..12'),
            ('2001-02-03 04:05:06', 'not a valid date'),
        )
        for value, error in data:
            with self.subTest(value=value):
                with self.assertRaisesRegex(ValueError, error):
                    parse_date(value)


class ParseDatetimeTest(SimpleTestCase):
    def test_valid(self):
        data = [
            # value, result
            ('2001-02-03 04:05:06', datetime(2001, 2, 3, 4, 5, 6)),
            ('2001-02-03 04:05:06+0000', datetime(2001, 2, 3, 4, 5, 6, tzinfo=UTC)),
            ('2001-02-03T04:05:06', datetime(2001, 2, 3, 4, 5, 6)),
        ]
        if django.VERSION >= (4, ):  # pragma: no cover
            data.append(('2001-02-03', datetime(2001, 2, 3, 0)))

        for value, result in data:
            with self.subTest(value=value):
                self.assertEqual(parse_datetime(value), result)

    def test_invalid(self):
        data = [
            # value, error
            ('', 'not a valid datetime'),
            ('JUNK', 'not a valid datetime'),
            ('2000-13-03 04:05:06', 'month must be in 1..12'),
        ]
        if django.VERSION < (4, ):  # pragma: no cover
            data.append(('2001-02-03', 'not a valid datetime'))
        for value, error in data:
            with self.subTest(value=value):
                with self.assertRaisesRegex(ValueError, error):
                    parse_datetime(value)
